# Какие бывают зависимости?

Для упрощения работы головы представим, что у нас есть:

~~~{.ruby}
class Worker
  def initalize(material)
    @material = material
  end

  def do_work(param)
    data = gets # Kernel#gets - обращение к методу данного объекта
    data = $stdin.gets # обращение к глобальной переменной $stdin, вызов её метода
    CSV.foreach('filename.csv') do |row| # обращение к методу класса
      # do something
    end
    @material.get_more # Обращение к методу внешнего объекта
    param.mutate # Аналогично вышепоказанному
    result = SomeMegaResult.new # Создать объект самостоятельно
    result.make_some_costing_operation
  end
end
~~~

* `gets` ведёт себя нестабильно (зависит от переданных приложению аргументов) => лучше не использовать
* `$stdin` - с такой завимотью "просто так" ничего не сделать => надо отказываться от глобальных переменных.

~~~
class MyCoolIO
  def initialize(stream = $stdin)
    @stream = stream
  end

  def gets
    @stream.gets
  end
end

class UsingThisIO
  def initialize(my_cool_io = $stdin)
    @my_cool_io
  end

  def do_some
    info = @my_cool_io.gets
  end
end
~~~

* Для решения проблемы зависимости от метода класса смотрим решение MiniTest/Stub

~~~
def test_stale_eh
  obj_under_test = Something.new
  refute obj_under_test.stale?

  Time.stub :now, Time.at(0) do
    assert obj_under_test.stale?
  end
end
~~~

При использовании библиотеки Mocha

~~~
# illustrates stubbing class method
  def test_should_count_number_of_orders_shipped_after_specified_date
    now = Time.now
    week_in_secs = 7 * 24 * 60 * 60
    order_1 = Order.new
    order_1.shipped_on = now - 1 * week_in_secs
    order_2 = Order.new
    order_2.shipped_on = now - 3 * week_in_secs
    Order.stubs(:find_all).returns([order_1, order_2])
    assert_equal 1, Order.number_shipped_since(now - 2 * week_in_secs)
  end
~~~

* Для решения проблемы зависимости от методов объектов
  * Да и пусть он будет.
  * Мы можем реализовать другой объект с необходимым нам интрефейсом. База данных / база данных в оперативной памяти. Журналирование в файл / журналирование в буфер. Допольнительный бонус: может пригодиться в других проектах.
  * Использовать объекты Mock из Minitest.
  * Использовать возможности библиотеки Mocha.

~~~
class MemeAsker
  def initialize(meme)
    @meme = meme
  end

  def ask(question)
    method = question.tr(" ", "_") + "?"
    @meme.__send__(method)
  end
end

require "minitest/autorun"

describe MemeAsker, :ask do
  describe "when passed an unpunctuated question" do
    it "should invoke the appropriate predicate method on the meme" do
      @meme = Minitest::Mock.new
      @meme_asker = MemeAsker.new @meme
      @meme.expect :will_it_blend?, :return_value

      @meme_asker.ask "will it blend"

      @meme.verify
    end
  end
end
~~~

# Проблема, которую попытаемся решить

Стоит задача в синхронизации Git-репозиториев с Github раз в день (ночью).

0. * Склонировать репозиторий, если это необходимо.
1. Получить все изменения из внешнего репозитория.
2. Оповестить систему управления задачами (Redmine) о том, что в репозитории появились новые наборы изменений.

Описание репозиториев будет находиться в yaml-файле.
